@@ -72,10 +72,10 @@ enum cyrusdb_dbflags {
 
 enum cyrusdb_openflags {
     CYRUSDB_CREATE    = 0x01,    /* Create the database if not existant */
-    CYRUSDB_MBOXSORT  = 0x02,    /* Use mailbox sort order ('.' sorts 1st) */
     CYRUSDB_CONVERT   = 0x04,    /* Convert to the named format if not already */
     CYRUSDB_NOCOMPACT = 0x08,    /* Don't run any database compaction routines */
-    CYRUSDB_SHARED    = 0x10     /* Open in shared lock mode */
+    CYRUSDB_SHARED    = 0x10,    /* Open in shared lock mode */
+    CYRUSDB_NOCRC     = 0x20     /* Don't check CRC32 on read */
 };
 
 typedef int foreach_p(void *rock,
@@ -104,9 +104,6 @@ struct cyrusdb_backend {
      * to reset state */
     int (*done)(void);
 
-    /* checkpoints this database environment */
-    int (*sync)(void);
-
     /* archives this database environment, and specified databases
      * into the specified directory */
     int (*archive)(const strarray_t *fnames, const char *dirname);
@@ -210,6 +207,9 @@ struct cyrusdb_backend {
                    struct txn **tid,
                    int force); /* 1 = ignore not found errors */
 
+    /* start a transaction (shared if flags & CYRUSDB_SHARED) */
+    int (*lock)(struct dbengine *db, struct txn **tid, int flags);
+
     /* Commit the transaction.  When commit() returns, the tid will no longer
      * be valid, regardless of if the commit succeeded or failed */
     int (*commit)(struct dbengine *db, struct txn *tid);
@@ -220,8 +220,7 @@ struct cyrusdb_backend {
     int (*dump)(struct dbengine *db, int detail);
     int (*consistent)(struct dbengine *db);
     int (*repack)(struct dbengine *db);
-    int (*compar)(struct dbengine *db, const char *s1, int l1,
-                  const char *s2, int l2);
+    int (*compar)(const char *s1, size_t l1, const char *s2, size_t l2);
 };
 
 extern int cyrusdb_copyfile(const char *srcname, const char *dstname);
@@ -288,18 +287,18 @@ extern int cyrusdb_store(struct db *db,
 extern int cyrusdb_delete(struct db *db,
                           const char *key, size_t keylen,
                           struct txn **tid, int force);
+extern int cyrusdb_lock(struct db *db, struct txn **tid, int flags);
 extern int cyrusdb_commit(struct db *db, struct txn *tid);
 extern int cyrusdb_abort(struct db *db, struct txn *tid);
 extern int cyrusdb_dump(struct db *db, int detail);
 extern int cyrusdb_consistent(struct db *db);
 extern int cyrusdb_repack(struct db *db);
 extern int cyrusdb_compar(struct db *db,
-                          const char *a, int alen,
-                          const char *b, int blen);
+                          const char *a, size_t alen,
+                          const char *b, size_t blen);
 
 /* somewhat special case, because they don't take a DB */
 
-extern int cyrusdb_sync(const char *backend);
 extern cyrusdb_archiver *cyrusdb_getarchiver(const char *backend);
 
 extern int cyrusdb_canfetchnext(const char *backend);
@@ -309,7 +308,6 @@ extern strarray_t *cyrusdb_backends(void);
 /* generic implementations */
 int cyrusdb_generic_init(const char *dbdir, int myflags);
 int cyrusdb_generic_done(void);
-int cyrusdb_generic_sync(void);
 int cyrusdb_generic_archive(const strarray_t *fnames, const char *dirname);
 int cyrusdb_generic_noarchive(const strarray_t *fnames, const char *dirname);
 int cyrusdb_generic_unlink(const char *fname, int flags);
